package main

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"strconv"
	"strings"

	"github.com/cosmos/cosmos-sdk/crypto/hd"
	cryptotypes "github.com/cosmos/cosmos-sdk/crypto/types"
	"github.com/cosmos/cosmos-sdk/simapp"
	"github.com/cosmos/cosmos-sdk/testutil/testdata"
	"github.com/cosmos/cosmos-sdk/types/tx/signing"
	xauthsigning "github.com/cosmos/cosmos-sdk/x/auth/signing"

	// "github.com/cosmos/cosmos-sdk/x/auth/tx"

	"github.com/cosmos/cosmos-sdk/client/tx"
	sdk "github.com/cosmos/cosmos-sdk/types"
	typestx "github.com/cosmos/cosmos-sdk/types/tx"

	avvytypes "github.com/cosmonaut/voter/x/voter/types"

	"google.golang.org/grpc"
)

type Seq struct {
	Height string    `json:"height"`
	Result SeqResult `json:"result"`
}

type SeqResult struct {
	Type  string        `json:"type"`
	Value SeqResultBody `json:"value"`
}

type SeqResultBody struct {
	Address    string           `json:"address"`
	Public_key SeqResultBodyPub `json:"public_key"`
	Sequence   string           `json:"sequence"`
}

type SeqResultBodyPub struct {
	Type  string `json:"type"`
	Value string `json:"value"`
}

func registerL3Node(l3_priv cryptotypes.PrivKey, l3node_pub cryptotypes.PubKey, l3_address sdk.AccAddress, options []string) error {

	fmt.Println("L3 Node address: ", l3_address.String())

	// TODO: нужно использовать app.AppCodec из cosmosapp
	encCfg := simapp.MakeTestEncodingConfig()

	// TODO забирать Passphrase из внешнего источника
	const (
		DefaultBIP39Passphrase = "attend canoe twenty shallow beyond permit purpose eyebrow collect bunker hotel already garment globe wrong sunset hair cycle economy unlock ride magic grocery alien"
	)

	// Создаём закрытый ключ Валидатора
	hdPath := hd.CreateHDPath(118, 0, 0).String()
	derivedPriv, errKeyDerive := hd.Secp256k1.Derive()(DefaultBIP39Passphrase, "", hdPath)
	if errKeyDerive != nil {
		println("errKeyDerive", errKeyDerive)
	}
	valPrivKey := hd.Secp256k1.Generate()(derivedPriv)
	valAddress := sdk.AccAddress(valPrivKey.PubKey().Address())
	creator := valAddress.String()

	fmt.Println("Validator address: ", creator)

	txBuilder := encCfg.TxConfig.NewTxBuilder()

	msg1 := avvytypes.NewMsgCreateL3Node(creator, l3node_pub.String(), options)

	errBult := txBuilder.SetMsgs(msg1)

	if errBult != nil {
		fmt.Println("txBuilder SetMsg ERROR: ", errBult)
		return errBult
	}

	// TODO: определить тип монеты, в которых рассчитывается комиссия
	var feeAmount = sdk.Coins{sdk.NewInt64Coin("foocoin", 0)}

	// TODO: найти метод для GasEstimate
	txBuilder.SetGasLimit(100000)
	txBuilder.SetFeeAmount(feeAmount)
	txBuilder.SetMemo("")
	// TODO: получить текущую высоту блока
	txBuilder.SetTimeoutHeight(73000)

	privs := []cryptotypes.PrivKey{valPrivKey}
	accNums := []uint64{0} // The accounts' account numbers

	senderSeq := getSequence(valAddress)
	number, errParse := strconv.ParseUint(string(senderSeq), 10, 64)
	if errParse != nil {
		fmt.Println("txBuilder SetMsg ERROR: ", errBult)
		return errParse
	}
	accSeqs := []uint64{number} // The accounts' sequence numbers

	// First round: we gather all the signer infos. We use the "set empty
	// signature" hack to do that.
	var sigsV2 []signing.SignatureV2
	for i, priv := range privs {
		sigV2 := signing.SignatureV2{
			PubKey: priv.PubKey(),
			Data: &signing.SingleSignatureData{
				SignMode:  encCfg.TxConfig.SignModeHandler().DefaultMode(),
				Signature: nil,
			},
			Sequence: accSeqs[i],
		}

		sigsV2 = append(sigsV2, sigV2)
	}
	res := txBuilder.SetSignatures(sigsV2...)
	fmt.Println("esimated gas used:", res) // Prints estimated gas used.

	// Second round: all signer infos are set, so each signer can sign.
	var chainID = "voter"

	sigsV2 = []signing.SignatureV2{}
	for i, priv := range privs {
		signerData := xauthsigning.SignerData{
			ChainID:       chainID,
			AccountNumber: accNums[i],
			Sequence:      accSeqs[i],
		}
		sigV2, errSignWithPK := tx.SignWithPrivKey(
			encCfg.TxConfig.SignModeHandler().DefaultMode(), signerData,
			txBuilder, priv, encCfg.TxConfig, accSeqs[i])
		if errSignWithPK != nil {
			fmt.Println("tx SignWithPrivKey ERROR: ", errSignWithPK)

			return errSignWithPK
		}

		sigsV2 = append(sigsV2, sigV2)
	}
	errSetSign := txBuilder.SetSignatures(sigsV2...)
	if errSetSign != nil {
		fmt.Println("txBuilder SetSignatures ERROR: ", errSetSign)

		return errSetSign
	}

	txBytes, errTxEncoder := encCfg.TxConfig.TxEncoder()(txBuilder.GetTx())
	if errTxEncoder != nil {
		fmt.Println("encCfg TxConfig TxEncoder ERROR: ", errTxEncoder)

		return errTxEncoder
	}

	// // Generate a JSON string.
	// txJSONBytes, err := encCfg.TxConfig.TxJSONEncoder()(txBuilder.GetTx())
	// if err != nil {
	// 	fmt.Println("encCfg TxConfig TxJSONEncoder ERROR: ", err)

	// 	return err
	// }
	// txJSON := string(txJSONBytes)
	// fmt.Println(txJSON)

	// Create a connection to the gRPC server.
	grpcConn, _ := grpc.Dial(
		"127.0.0.1:9090",    // Or your gRPC server address.
		grpc.WithInsecure(), // The Cosmos SDK doesn't support any transport security mechanism.
	)
	defer grpcConn.Close()

	// Broadcast the tx via gRPC. We create a new client for the Protobuf Tx
	// service.
	txClient := typestx.NewServiceClient(grpcConn)
	// We then call the BroadcastTx method on this client.
	grpcRes, errBroadcastTx := txClient.BroadcastTx(
		context.Background(),
		&typestx.BroadcastTxRequest{
			Mode:    typestx.BroadcastMode_BROADCAST_MODE_SYNC,
			TxBytes: txBytes, // Proto-binary of the signed transaction, see previous step.
		},
	)
	if errBroadcastTx != nil {
		fmt.Println("txClient BroadcastTx ERROR: ", errBroadcastTx)

		return errBroadcastTx
	}

	fmt.Println("grpcRes.TxResponse: ", grpcRes.TxResponse)

	return nil
}

func registerRollup(prev_hash string, current_hash string, data_source_uri string, txs []string) error {

	// TODO: нужно использовать app.AppCodec из cosmosapp
	encCfg := simapp.MakeTestEncodingConfig()

	// TODO забирать Passphrase из внешнего источника
	const (
		DefaultBIP39Passphrase = "attend canoe twenty shallow beyond permit purpose eyebrow collect bunker hotel already garment globe wrong sunset hair cycle economy unlock ride magic grocery alien"
	)

	// Создаём закрытый ключ Валидатора
	hdPath := hd.CreateHDPath(118, 0, 0).String()
	derivedPriv, errKeyDerive := hd.Secp256k1.Derive()(DefaultBIP39Passphrase, "", hdPath)
	if errKeyDerive != nil {
		println("errKeyDerive", errKeyDerive)
	}
	valPrivKey := hd.Secp256k1.Generate()(derivedPriv)
	valAddress := sdk.AccAddress(valPrivKey.PubKey().Address())
	creator := valAddress.String()

	txBuilder := encCfg.TxConfig.NewTxBuilder()

	msg1 := avvytypes.NewMsgCreateRollup(creator, prev_hash, current_hash, data_source_uri, txs)

	errBult := txBuilder.SetMsgs(msg1)

	if errBult != nil {
		fmt.Println("txBuilder SetMsg ERROR: ", errBult)
		return errBult
	}

	// TODO: определить тип монеты, в которых рассчитывается комиссия
	var feeAmount = sdk.Coins{sdk.NewInt64Coin("foocoin", 0)}

	// TODO: найти метод для GasEstimate
	txBuilder.SetGasLimit(100000)
	txBuilder.SetFeeAmount(feeAmount)
	txBuilder.SetMemo("")
	// TODO: получить текущую высоту блока
	txBuilder.SetTimeoutHeight(73000)

	privs := []cryptotypes.PrivKey{valPrivKey}
	accNums := []uint64{0} // The accounts' account numbers

	senderSeq := getSequence(valAddress)
	number, errParse := strconv.ParseUint(string(senderSeq), 10, 64)
	if errParse != nil {
		fmt.Println("txBuilder SetMsg ERROR: ", errBult)
		return errParse
	}
	accSeqs := []uint64{number} // The accounts' sequence numbers

	// First round: we gather all the signer infos. We use the "set empty
	// signature" hack to do that.
	var sigsV2 []signing.SignatureV2
	for i, priv := range privs {
		sigV2 := signing.SignatureV2{
			PubKey: priv.PubKey(),
			Data: &signing.SingleSignatureData{
				SignMode:  encCfg.TxConfig.SignModeHandler().DefaultMode(),
				Signature: nil,
			},
			Sequence: accSeqs[i],
		}

		sigsV2 = append(sigsV2, sigV2)
	}
	res := txBuilder.SetSignatures(sigsV2...)
	fmt.Println("esimated gas used:", res) // Prints estimated gas used.

	// Second round: all signer infos are set, so each signer can sign.
	var chainID = "voter"

	sigsV2 = []signing.SignatureV2{}
	for i, priv := range privs {
		signerData := xauthsigning.SignerData{
			ChainID:       chainID,
			AccountNumber: accNums[i],
			Sequence:      accSeqs[i],
		}
		sigV2, errSignWithPK := tx.SignWithPrivKey(
			encCfg.TxConfig.SignModeHandler().DefaultMode(), signerData,
			txBuilder, priv, encCfg.TxConfig, accSeqs[i])
		if errSignWithPK != nil {
			fmt.Println("tx SignWithPrivKey ERROR: ", errSignWithPK)

			return errSignWithPK
		}

		sigsV2 = append(sigsV2, sigV2)
	}
	errSetSign := txBuilder.SetSignatures(sigsV2...)
	if errSetSign != nil {
		fmt.Println("txBuilder SetSignatures ERROR: ", errSetSign)

		return errSetSign
	}

	txBytes, errTxEncoder := encCfg.TxConfig.TxEncoder()(txBuilder.GetTx())
	if errTxEncoder != nil {
		fmt.Println("encCfg TxConfig TxEncoder ERROR: ", errTxEncoder)

		return errTxEncoder
	}

	// // Generate a JSON string.
	// txJSONBytes, err := encCfg.TxConfig.TxJSONEncoder()(txBuilder.GetTx())
	// if err != nil {
	// 	fmt.Println("encCfg TxConfig TxJSONEncoder ERROR: ", err)

	// 	return err
	// }
	// txJSON := string(txJSONBytes)
	// fmt.Println(txJSON)

	// Create a connection to the gRPC server.
	grpcConn, _ := grpc.Dial(
		"0.0.0.0:9090",      // Or your gRPC server address.
		grpc.WithInsecure(), // The Cosmos SDK doesn't support any transport security mechanism.
	)
	defer grpcConn.Close()

	// Broadcast the tx via gRPC. We create a new client for the Protobuf Tx
	// service.
	txClient := typestx.NewServiceClient(grpcConn)
	// We then call the BroadcastTx method on this client.
	grpcRes, errBroadcastTx := txClient.BroadcastTx(
		context.Background(),
		&typestx.BroadcastTxRequest{
			Mode:    typestx.BroadcastMode_BROADCAST_MODE_SYNC,
			TxBytes: txBytes, // Proto-binary of the signed transaction, see previous step.
		},
	)
	if errBroadcastTx != nil {
		fmt.Println("txClient BroadcastTx ERROR: ", errBroadcastTx)

		return errBroadcastTx
	}

	fmt.Println("grpcRes.TxResponse: ", grpcRes.TxResponse)

	return nil
}

func faucet(addr sdk.AccAddress) {
	values := map[string]string{"address": addr.String(), "coins": "100foocoin"}
	json_data, err := json.Marshal(values)

	resp, err := http.Post("http://0.0.0.0:4500/", "application/json", bytes.NewBuffer(json_data))

	if err != nil {
		log.Fatal(err)
	}

	fmt.Println("resp: ", resp)
}

func getSequence(addr sdk.AccAddress) string {

	builder := strings.Builder{}
	builder.WriteString("http://0.0.0.0:1317/auth/accounts/")
	builder.WriteString(addr.String())

	fmt.Println(builder.String())

	resp, err := http.Get(builder.String())

	if err != nil {
		log.Fatal(err)
	}

	defer resp.Body.Close()

	body, err := ioutil.ReadAll(resp.Body)

	if err != nil {
		log.Fatal(err)
	}

	textBytes := []byte(body)

	seq1 := Seq{}
	errUnmarshal := json.Unmarshal(textBytes, &seq1)

	if errUnmarshal != nil {
		fmt.Println(errUnmarshal)
	}

	fmt.Println("seq1: ", seq1)

	return seq1.Result.Value.Sequence
}

func main() {
	fmt.Println("\nStart\n")
	privKey1, pubKey1, addr1 := testdata.KeyTestPubAddr()
	registerL3Node(privKey1, pubKey1, addr1, []string{"options1", "options2"})
	registerRollup("prev_hash", "current_hash", "dsu", []string{"tx1", "tx2", "tx3"})
	fmt.Println("End")
}
