export interface ProtobufAny {
    "@type"?: string;
}
export interface RpcStatus {
    /** @format int32 */
    code?: number;
    message?: string;
    details?: ProtobufAny[];
}
/**
* message SomeRequest {
         Foo some_parameter = 1;
         PageRequest pagination = 2;
 }
*/
export interface V1Beta1PageRequest {
    /**
     * key is a value returned in PageResponse.next_key to begin
     * querying the next page most efficiently. Only one of offset or key
     * should be set.
     * @format byte
     */
    key?: string;
    /**
     * offset is a numeric offset that can be used when key is unavailable.
     * It is less efficient than using key. Only one of offset or key should
     * be set.
     * @format uint64
     */
    offset?: string;
    /**
     * limit is the total number of results to be returned in the result page.
     * If left empty it will default to a value to be set by each app.
     * @format uint64
     */
    limit?: string;
    /**
     * count_total is set to true  to indicate that the result set should include
     * a count of the total number of items available for pagination in UIs.
     * count_total is only respected when offset is used. It is ignored when key
     * is set.
     */
    countTotal?: boolean;
    /** reverse is set to true if results are to be returned in the descending order. */
    reverse?: boolean;
}
/**
* PageResponse is to be embedded in gRPC response messages where the
corresponding request message has used PageRequest.

 message SomeResponse {
         repeated Bar results = 1;
         PageResponse page = 2;
 }
*/
export interface V1Beta1PageResponse {
    /** @format byte */
    nextKey?: string;
    /** @format uint64 */
    total?: string;
}
export interface VoterAttachedUser {
    /** @format int32 */
    iidapId?: number;
    /** @format int32 */
    account?: number;
    userName?: string;
    creator?: string;
}
export interface VoterL3Node {
    /** @format uint64 */
    id?: string;
    publicKey?: string;
    options?: string[];
    creator?: string;
}
export interface VoterMessage {
    /** @format uint64 */
    id?: string;
    status?: string;
    name?: string;
    data?: string[];
    creator?: string;
}
export declare type VoterMsgCreateAttachedUserResponse = object;
export interface VoterMsgCreateL3NodeResponse {
    /** @format uint64 */
    id?: string;
}
export interface VoterMsgCreateMessageResponse {
    /** @format uint64 */
    id?: string;
}
export interface VoterMsgCreateRollupDigestResponse {
    /** @format uint64 */
    id?: string;
}
export interface VoterMsgCreateRollupResponse {
    /** @format uint64 */
    id?: string;
}
export declare type VoterMsgDeleteAttachedUserResponse = object;
export declare type VoterMsgDeleteL3NodeResponse = object;
export declare type VoterMsgDeleteMessageResponse = object;
export declare type VoterMsgDeleteRollupDigestResponse = object;
export declare type VoterMsgDeleteRollupResponse = object;
export declare type VoterMsgUpdateAttachedUserResponse = object;
export declare type VoterMsgUpdateL3NodeResponse = object;
export declare type VoterMsgUpdateMessageResponse = object;
export declare type VoterMsgUpdateRollupDigestResponse = object;
export declare type VoterMsgUpdateRollupResponse = object;
export interface VoterQueryAllAttachedUserResponse {
    attachedUser?: VoterAttachedUser[];
    /**
     * PageResponse is to be embedded in gRPC response messages where the
     * corresponding request message has used PageRequest.
     *
     *  message SomeResponse {
     *          repeated Bar results = 1;
     *          PageResponse page = 2;
     *  }
     */
    pagination?: V1Beta1PageResponse;
}
export interface VoterQueryAllL3NodeResponse {
    L3Node?: VoterL3Node[];
    /**
     * PageResponse is to be embedded in gRPC response messages where the
     * corresponding request message has used PageRequest.
     *
     *  message SomeResponse {
     *          repeated Bar results = 1;
     *          PageResponse page = 2;
     *  }
     */
    pagination?: V1Beta1PageResponse;
}
export interface VoterQueryAllMessageResponse {
    Message?: VoterMessage[];
    /**
     * PageResponse is to be embedded in gRPC response messages where the
     * corresponding request message has used PageRequest.
     *
     *  message SomeResponse {
     *          repeated Bar results = 1;
     *          PageResponse page = 2;
     *  }
     */
    pagination?: V1Beta1PageResponse;
}
export interface VoterQueryAllRollupDigestResponse {
    RollupDigest?: VoterRollupDigest[];
    /**
     * PageResponse is to be embedded in gRPC response messages where the
     * corresponding request message has used PageRequest.
     *
     *  message SomeResponse {
     *          repeated Bar results = 1;
     *          PageResponse page = 2;
     *  }
     */
    pagination?: V1Beta1PageResponse;
}
export interface VoterQueryAllRollupResponse {
    Rollup?: VoterRollup[];
    /**
     * PageResponse is to be embedded in gRPC response messages where the
     * corresponding request message has used PageRequest.
     *
     *  message SomeResponse {
     *          repeated Bar results = 1;
     *          PageResponse page = 2;
     *  }
     */
    pagination?: V1Beta1PageResponse;
}
export interface VoterQueryGetAttachedUserResponse {
    attachedUser?: VoterAttachedUser;
}
export interface VoterQueryGetL3NodeResponse {
    L3Node?: VoterL3Node;
}
export interface VoterQueryGetMessageResponse {
    Message?: VoterMessage;
}
export interface VoterQueryGetRollupDigestResponse {
    RollupDigest?: VoterRollupDigest;
}
export interface VoterQueryGetRollupResponse {
    Rollup?: VoterRollup;
}
export interface VoterRollup {
    /** @format uint64 */
    id?: string;
    prevHash?: string;
    currentHash?: string;
    dsUri?: string;
    inbox?: string[];
    creator?: string;
}
export interface VoterRollupDigest {
    /** @format uint64 */
    id?: string;
    dsUri?: string;
    currentHash?: string;
    status?: string;
    creator?: string;
}
export declare type QueryParamsType = Record<string | number, any>;
export declare type ResponseFormat = keyof Omit<Body, "body" | "bodyUsed">;
export interface FullRequestParams extends Omit<RequestInit, "body"> {
    /** set parameter to `true` for call `securityWorker` for this request */
    secure?: boolean;
    /** request path */
    path: string;
    /** content type of request body */
    type?: ContentType;
    /** query params */
    query?: QueryParamsType;
    /** format of response (i.e. response.json() -> format: "json") */
    format?: keyof Omit<Body, "body" | "bodyUsed">;
    /** request body */
    body?: unknown;
    /** base url */
    baseUrl?: string;
    /** request cancellation token */
    cancelToken?: CancelToken;
}
export declare type RequestParams = Omit<FullRequestParams, "body" | "method" | "query" | "path">;
export interface ApiConfig<SecurityDataType = unknown> {
    baseUrl?: string;
    baseApiParams?: Omit<RequestParams, "baseUrl" | "cancelToken" | "signal">;
    securityWorker?: (securityData: SecurityDataType) => RequestParams | void;
}
export interface HttpResponse<D extends unknown, E extends unknown = unknown> extends Response {
    data: D;
    error: E;
}
declare type CancelToken = Symbol | string | number;
export declare enum ContentType {
    Json = "application/json",
    FormData = "multipart/form-data",
    UrlEncoded = "application/x-www-form-urlencoded"
}
export declare class HttpClient<SecurityDataType = unknown> {
    baseUrl: string;
    private securityData;
    private securityWorker;
    private abortControllers;
    private baseApiParams;
    constructor(apiConfig?: ApiConfig<SecurityDataType>);
    setSecurityData: (data: SecurityDataType) => void;
    private addQueryParam;
    protected toQueryString(rawQuery?: QueryParamsType): string;
    protected addQueryParams(rawQuery?: QueryParamsType): string;
    private contentFormatters;
    private mergeRequestParams;
    private createAbortSignal;
    abortRequest: (cancelToken: CancelToken) => void;
    request: <T = any, E = any>({ body, secure, path, type, query, format, baseUrl, cancelToken, ...params }: FullRequestParams) => Promise<HttpResponse<T, E>>;
}
/**
 * @title voter/attached_user.proto
 * @version version not set
 */
export declare class Api<SecurityDataType extends unknown> extends HttpClient<SecurityDataType> {
    /**
     * No description
     *
     * @tags Query
     * @name QueryAttachedUserAll
     * @summary Queries a list of attachedUser items.
     * @request GET:/cosmonaut/voter/voter/attachedUser
     */
    queryAttachedUserAll: (query?: {
        "pagination.key"?: string;
        "pagination.offset"?: string;
        "pagination.limit"?: string;
        "pagination.countTotal"?: boolean;
        "pagination.reverse"?: boolean;
    }, params?: RequestParams) => Promise<HttpResponse<VoterQueryAllAttachedUserResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryAttachedUser
     * @summary Queries a attachedUser by index.
     * @request GET:/cosmonaut/voter/voter/attachedUser/{iidapId}/{account}
     */
    queryAttachedUser: (iidapId: number, account: number, params?: RequestParams) => Promise<HttpResponse<VoterQueryGetAttachedUserResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryL3NodeAll
     * @summary Queries a list of l3Node items.
     * @request GET:/cosmonaut/voter/voter/l3Node
     */
    queryL3NodeAll: (query?: {
        "pagination.key"?: string;
        "pagination.offset"?: string;
        "pagination.limit"?: string;
        "pagination.countTotal"?: boolean;
        "pagination.reverse"?: boolean;
    }, params?: RequestParams) => Promise<HttpResponse<VoterQueryAllL3NodeResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryL3Node
     * @summary Queries a l3Node by id.
     * @request GET:/cosmonaut/voter/voter/l3Node/{id}
     */
    queryL3Node: (id: string, params?: RequestParams) => Promise<HttpResponse<VoterQueryGetL3NodeResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryMessageAll
     * @summary Queries a list of message items.
     * @request GET:/cosmonaut/voter/voter/message
     */
    queryMessageAll: (query?: {
        "pagination.key"?: string;
        "pagination.offset"?: string;
        "pagination.limit"?: string;
        "pagination.countTotal"?: boolean;
        "pagination.reverse"?: boolean;
    }, params?: RequestParams) => Promise<HttpResponse<VoterQueryAllMessageResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryMessage
     * @summary Queries a message by id.
     * @request GET:/cosmonaut/voter/voter/message/{id}
     */
    queryMessage: (id: string, params?: RequestParams) => Promise<HttpResponse<VoterQueryGetMessageResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryRollupAll
     * @summary Queries a list of rollup items.
     * @request GET:/cosmonaut/voter/voter/rollup
     */
    queryRollupAll: (query?: {
        "pagination.key"?: string;
        "pagination.offset"?: string;
        "pagination.limit"?: string;
        "pagination.countTotal"?: boolean;
        "pagination.reverse"?: boolean;
    }, params?: RequestParams) => Promise<HttpResponse<VoterQueryAllRollupResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryRollup
     * @summary Queries a rollup by id.
     * @request GET:/cosmonaut/voter/voter/rollup/{id}
     */
    queryRollup: (id: string, params?: RequestParams) => Promise<HttpResponse<VoterQueryGetRollupResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryRollupDigestAll
     * @summary Queries a list of rollupDigest items.
     * @request GET:/cosmonaut/voter/voter/rollupDigest
     */
    queryRollupDigestAll: (query?: {
        "pagination.key"?: string;
        "pagination.offset"?: string;
        "pagination.limit"?: string;
        "pagination.countTotal"?: boolean;
        "pagination.reverse"?: boolean;
    }, params?: RequestParams) => Promise<HttpResponse<VoterQueryAllRollupDigestResponse, RpcStatus>>;
    /**
     * No description
     *
     * @tags Query
     * @name QueryRollupDigest
     * @summary Queries a rollupDigest by id.
     * @request GET:/cosmonaut/voter/voter/rollupDigest/{id}
     */
    queryRollupDigest: (id: string, params?: RequestParams) => Promise<HttpResponse<VoterQueryGetRollupDigestResponse, RpcStatus>>;
}
export {};
