package cli

import (
	"context"

	"github.com/spf13/cast"
	"github.com/spf13/cobra"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/client"
	"github.com/cosmos/cosmos-sdk/client/flags"
)

func CmdListAttachedUser() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "list-attached-user",
		Short: "list all attachedUser",
		RunE: func(cmd *cobra.Command, args []string) error {
			clientCtx := client.GetClientContextFromCmd(cmd)

			pageReq, err := client.ReadPageRequest(cmd.Flags())
			if err != nil {
				return err
			}

			queryClient := types.NewQueryClient(clientCtx)

			params := &types.QueryAllAttachedUserRequest{
				Pagination: pageReq,
			}

			res, err := queryClient.AttachedUserAll(context.Background(), params)
			if err != nil {
				return err
			}

			return clientCtx.PrintProto(res)
		},
	}

	flags.AddPaginationFlagsToCmd(cmd, cmd.Use)
	flags.AddQueryFlagsToCmd(cmd)

	return cmd
}

func CmdShowAttachedUser() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "show-attached-user [iidap-id] [account]",
		Short: "shows a attachedUser",
		Args:  cobra.ExactArgs(2),
		RunE: func(cmd *cobra.Command, args []string) (err error) {
			clientCtx := client.GetClientContextFromCmd(cmd)

			queryClient := types.NewQueryClient(clientCtx)

			argIidapId, err := cast.ToInt32E(args[0])
			if err != nil {
				return err
			}
			argAccount, err := cast.ToInt32E(args[1])
			if err != nil {
				return err
			}

			params := &types.QueryGetAttachedUserRequest{
				IidapId: argIidapId,
				Account: argAccount,
			}

			res, err := queryClient.AttachedUser(context.Background(), params)
			if err != nil {
				return err
			}

			return clientCtx.PrintProto(res)
		},
	}

	flags.AddQueryFlagsToCmd(cmd)

	return cmd
}
