package cli

import (
	"github.com/spf13/cast"
	"github.com/spf13/cobra"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/client"
	"github.com/cosmos/cosmos-sdk/client/flags"
	"github.com/cosmos/cosmos-sdk/client/tx"
)

func CmdCreateAttachedUser() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "create-attached-user [iidap-id] [account] [user-name]",
		Short: "Create a new attachedUser",
		Args:  cobra.ExactArgs(3),
		RunE: func(cmd *cobra.Command, args []string) (err error) {
			// Get indexes
			indexIidapId, err := cast.ToInt32E(args[0])
			if err != nil {
				return err
			}
			indexAccount, err := cast.ToInt32E(args[1])
			if err != nil {
				return err
			}

			// Get value arguments
			argUserName := args[2]

			clientCtx, err := client.GetClientTxContext(cmd)
			if err != nil {
				return err
			}

			msg := types.NewMsgCreateAttachedUser(
				clientCtx.GetFromAddress().String(),
				indexIidapId,
				indexAccount,
				argUserName,
			)
			if err := msg.ValidateBasic(); err != nil {
				return err
			}
			return tx.GenerateOrBroadcastTxCLI(clientCtx, cmd.Flags(), msg)
		},
	}

	flags.AddTxFlagsToCmd(cmd)

	return cmd
}

func CmdUpdateAttachedUser() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "update-attached-user [iidap-id] [account] [user-name]",
		Short: "Update a attachedUser",
		Args:  cobra.ExactArgs(3),
		RunE: func(cmd *cobra.Command, args []string) (err error) {
			// Get indexes
			indexIidapId, err := cast.ToInt32E(args[0])
			if err != nil {
				return err
			}
			indexAccount, err := cast.ToInt32E(args[1])
			if err != nil {
				return err
			}

			// Get value arguments
			argUserName := args[2]

			clientCtx, err := client.GetClientTxContext(cmd)
			if err != nil {
				return err
			}

			msg := types.NewMsgUpdateAttachedUser(
				clientCtx.GetFromAddress().String(),
				indexIidapId,
				indexAccount,
				argUserName,
			)
			if err := msg.ValidateBasic(); err != nil {
				return err
			}
			return tx.GenerateOrBroadcastTxCLI(clientCtx, cmd.Flags(), msg)
		},
	}

	flags.AddTxFlagsToCmd(cmd)

	return cmd
}

func CmdDeleteAttachedUser() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "delete-attached-user [iidap-id] [account]",
		Short: "Delete a attachedUser",
		Args:  cobra.ExactArgs(2),
		RunE: func(cmd *cobra.Command, args []string) (err error) {
			indexIidapId, err := cast.ToInt32E(args[0])
			if err != nil {
				return err
			}
			indexAccount, err := cast.ToInt32E(args[1])
			if err != nil {
				return err
			}

			clientCtx, err := client.GetClientTxContext(cmd)
			if err != nil {
				return err
			}

			msg := types.NewMsgDeleteAttachedUser(
				clientCtx.GetFromAddress().String(),
				indexIidapId,
				indexAccount,
			)
			if err := msg.ValidateBasic(); err != nil {
				return err
			}
			return tx.GenerateOrBroadcastTxCLI(clientCtx, cmd.Flags(), msg)
		},
	}

	flags.AddTxFlagsToCmd(cmd)

	return cmd
}
