package cli

import (
	"strconv"

	"github.com/spf13/cobra"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/client"
	"github.com/cosmos/cosmos-sdk/client/flags"
	"github.com/cosmos/cosmos-sdk/client/tx"
)

func CmdCreateRollup() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "create-rollup [prev-hash] [current-hash] [ds-uri] [inbox]",
		Short: "Create a new rollup",
		Args:  cobra.MinimumNArgs(4),
		RunE: func(cmd *cobra.Command, args []string) (err error) {
			argPrevHash := args[0]
			argCurrentHash := args[1]
			argDsUri := args[2]
			argInbox := args[3:len(args)]

			clientCtx, err := client.GetClientTxContext(cmd)
			if err != nil {
				return err
			}

			msg := types.NewMsgCreateRollup(clientCtx.GetFromAddress().String(), argPrevHash, argCurrentHash, argDsUri, argInbox)
			if err := msg.ValidateBasic(); err != nil {
				return err
			}
			return tx.GenerateOrBroadcastTxCLI(clientCtx, cmd.Flags(), msg)
		},
	}

	flags.AddTxFlagsToCmd(cmd)

	return cmd
}

func CmdUpdateRollup() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "update-rollup [id] [prev-hash] [current-hash] [ds-uri] [inbox]",
		Short: "Update a rollup",
		Args:  cobra.MinimumNArgs(5),
		RunE: func(cmd *cobra.Command, args []string) (err error) {
			id, err := strconv.ParseUint(args[0], 10, 64)
			if err != nil {
				return err
			}

			argPrevHash := args[1]

			argCurrentHash := args[2]

			argDsUri := args[3]

			argInbox := args[4:len(args)]

			clientCtx, err := client.GetClientTxContext(cmd)
			if err != nil {
				return err
			}

			msg := types.NewMsgUpdateRollup(clientCtx.GetFromAddress().String(), id, argPrevHash, argCurrentHash, argDsUri, argInbox)
			if err := msg.ValidateBasic(); err != nil {
				return err
			}
			return tx.GenerateOrBroadcastTxCLI(clientCtx, cmd.Flags(), msg)
		},
	}

	flags.AddTxFlagsToCmd(cmd)

	return cmd
}

func CmdDeleteRollup() *cobra.Command {
	cmd := &cobra.Command{
		Use:   "delete-rollup [id]",
		Short: "Delete a rollup by id",
		Args:  cobra.ExactArgs(1),
		RunE: func(cmd *cobra.Command, args []string) error {
			id, err := strconv.ParseUint(args[0], 10, 64)
			if err != nil {
				return err
			}

			clientCtx, err := client.GetClientTxContext(cmd)
			if err != nil {
				return err
			}

			msg := types.NewMsgDeleteRollup(clientCtx.GetFromAddress().String(), id)
			if err := msg.ValidateBasic(); err != nil {
				return err
			}
			return tx.GenerateOrBroadcastTxCLI(clientCtx, cmd.Flags(), msg)
		},
	}

	flags.AddTxFlagsToCmd(cmd)

	return cmd
}
