package voter

import (
	"github.com/cosmonaut/voter/x/voter/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
)

// InitGenesis initializes the capability module's state from a provided genesis
// state.
func InitGenesis(ctx sdk.Context, k keeper.Keeper, genState types.GenesisState) {
	// Set all the rollup
	for _, elem := range genState.RollupList {
		k.SetRollup(ctx, elem)
	}

	// Set rollup count
	k.SetRollupCount(ctx, genState.RollupCount)
	// Set all the message
	for _, elem := range genState.MessageList {
		k.SetMessage(ctx, elem)
	}

	// Set message count
	k.SetMessageCount(ctx, genState.MessageCount)
	// Set all the l3Node
	for _, elem := range genState.L3NodeList {
		k.SetL3Node(ctx, elem)
	}

	// Set l3Node count
	k.SetL3NodeCount(ctx, genState.L3NodeCount)
	// Set all the attachedUser
	for _, elem := range genState.AttachedUserList {
		k.SetAttachedUser(ctx, elem)
	}
	// Set all the rollupDigest
	for _, elem := range genState.RollupDigestList {
		k.SetRollupDigest(ctx, elem)
	}

	// Set rollupDigest count
	k.SetRollupDigestCount(ctx, genState.RollupDigestCount)
	// this line is used by starport scaffolding # genesis/module/init
}

// ExportGenesis returns the capability module's exported genesis.
func ExportGenesis(ctx sdk.Context, k keeper.Keeper) *types.GenesisState {
	genesis := types.DefaultGenesis()

	genesis.RollupList = k.GetAllRollup(ctx)
	genesis.RollupCount = k.GetRollupCount(ctx)
	genesis.MessageList = k.GetAllMessage(ctx)
	genesis.MessageCount = k.GetMessageCount(ctx)
	genesis.L3NodeList = k.GetAllL3Node(ctx)
	genesis.L3NodeCount = k.GetL3NodeCount(ctx)
	genesis.AttachedUserList = k.GetAllAttachedUser(ctx)
	genesis.RollupDigestList = k.GetAllRollupDigest(ctx)
	genesis.RollupDigestCount = k.GetRollupDigestCount(ctx)
	// this line is used by starport scaffolding # genesis/module/export

	return genesis
}
