package keeper_test

import (
	"strconv"
	"testing"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	"github.com/stretchr/testify/require"
)

// Prevent strconv unused error
var _ = strconv.IntSize

func createNAttachedUser(keeper *keeper.Keeper, ctx sdk.Context, n int) []types.AttachedUser {
	items := make([]types.AttachedUser, n)
	for i := range items {
		items[i].IidapId = int32(i)
		items[i].Account = int32(i)

		keeper.SetAttachedUser(ctx, items[i])
	}
	return items
}

func TestAttachedUserGet(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNAttachedUser(keeper, ctx, 10)
	for _, item := range items {
		rst, found := keeper.GetAttachedUser(ctx,
			item.IidapId,
			item.Account,
		)
		require.True(t, found)
		require.Equal(t, item, rst)
	}
}
func TestAttachedUserRemove(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNAttachedUser(keeper, ctx, 10)
	for _, item := range items {
		keeper.RemoveAttachedUser(ctx,
			item.IidapId,
			item.Account,
		)
		_, found := keeper.GetAttachedUser(ctx,
			item.IidapId,
			item.Account,
		)
		require.False(t, found)
	}
}

func TestAttachedUserGetAll(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNAttachedUser(keeper, ctx, 10)
	require.ElementsMatch(t, items, keeper.GetAllAttachedUser(ctx))
}
