package keeper

import (
	"context"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/cosmos/cosmos-sdk/types/query"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

func (k Keeper) L3NodeAll(c context.Context, req *types.QueryAllL3NodeRequest) (*types.QueryAllL3NodeResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	var l3Nodes []types.L3Node
	ctx := sdk.UnwrapSDKContext(c)

	store := ctx.KVStore(k.storeKey)
	l3NodeStore := prefix.NewStore(store, types.KeyPrefix(types.L3NodeKey))

	pageRes, err := query.Paginate(l3NodeStore, req.Pagination, func(key []byte, value []byte) error {
		var l3Node types.L3Node
		if err := k.cdc.Unmarshal(value, &l3Node); err != nil {
			return err
		}

		l3Nodes = append(l3Nodes, l3Node)
		return nil
	})

	if err != nil {
		return nil, status.Error(codes.Internal, err.Error())
	}

	return &types.QueryAllL3NodeResponse{L3Node: l3Nodes, Pagination: pageRes}, nil
}

func (k Keeper) L3Node(c context.Context, req *types.QueryGetL3NodeRequest) (*types.QueryGetL3NodeResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	ctx := sdk.UnwrapSDKContext(c)
	l3Node, found := k.GetL3Node(ctx, req.Id)
	if !found {
		return nil, sdkerrors.ErrKeyNotFound
	}

	return &types.QueryGetL3NodeResponse{L3Node: l3Node}, nil
}
