package keeper

import (
	"context"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/cosmos/cosmos-sdk/types/query"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

func (k Keeper) RollupDigestAll(c context.Context, req *types.QueryAllRollupDigestRequest) (*types.QueryAllRollupDigestResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	var rollupDigests []types.RollupDigest
	ctx := sdk.UnwrapSDKContext(c)

	store := ctx.KVStore(k.storeKey)
	rollupDigestStore := prefix.NewStore(store, types.KeyPrefix(types.RollupDigestKey))

	pageRes, err := query.Paginate(rollupDigestStore, req.Pagination, func(key []byte, value []byte) error {
		var rollupDigest types.RollupDigest
		if err := k.cdc.Unmarshal(value, &rollupDigest); err != nil {
			return err
		}

		rollupDigests = append(rollupDigests, rollupDigest)
		return nil
	})

	if err != nil {
		return nil, status.Error(codes.Internal, err.Error())
	}

	return &types.QueryAllRollupDigestResponse{RollupDigest: rollupDigests, Pagination: pageRes}, nil
}

func (k Keeper) RollupDigest(c context.Context, req *types.QueryGetRollupDigestRequest) (*types.QueryGetRollupDigestResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	ctx := sdk.UnwrapSDKContext(c)
	rollupDigest, found := k.GetRollupDigest(ctx, req.Id)
	if !found {
		return nil, sdkerrors.ErrKeyNotFound
	}

	return &types.QueryGetRollupDigestResponse{RollupDigest: rollupDigest}, nil
}
