package keeper_test

import (
	"testing"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	"github.com/stretchr/testify/require"
)

func createNL3Node(keeper *keeper.Keeper, ctx sdk.Context, n int) []types.L3Node {
	items := make([]types.L3Node, n)
	for i := range items {
		items[i].Id = keeper.AppendL3Node(ctx, items[i])
	}
	return items
}

func TestL3NodeGet(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNL3Node(keeper, ctx, 10)
	for _, item := range items {
		got, found := keeper.GetL3Node(ctx, item.Id)
		require.True(t, found)
		require.Equal(t, item, got)
	}
}

func TestL3NodeRemove(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNL3Node(keeper, ctx, 10)
	for _, item := range items {
		keeper.RemoveL3Node(ctx, item.Id)
		_, found := keeper.GetL3Node(ctx, item.Id)
		require.False(t, found)
	}
}

func TestL3NodeGetAll(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNL3Node(keeper, ctx, 10)
	require.ElementsMatch(t, items, keeper.GetAllL3Node(ctx))
}

func TestL3NodeCount(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNL3Node(keeper, ctx, 10)
	count := uint64(len(items))
	require.Equal(t, count, keeper.GetL3NodeCount(ctx))
}
