package keeper_test

import (
	"testing"

	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/stretchr/testify/require"

	"github.com/cosmonaut/voter/x/voter/types"
)

func TestL3NodeMsgServerCreate(t *testing.T) {
	srv, ctx := setupMsgServer(t)
	creator := "A"
	for i := 0; i < 5; i++ {
		resp, err := srv.CreateL3Node(ctx, &types.MsgCreateL3Node{Creator: creator})
		require.NoError(t, err)
		require.Equal(t, i, int(resp.Id))
	}
}

func TestL3NodeMsgServerUpdate(t *testing.T) {
	creator := "A"

	for _, tc := range []struct {
		desc    string
		request *types.MsgUpdateL3Node
		err     error
	}{
		{
			desc:    "Completed",
			request: &types.MsgUpdateL3Node{Creator: creator},
		},
		{
			desc:    "Unauthorized",
			request: &types.MsgUpdateL3Node{Creator: "B"},
			err:     sdkerrors.ErrUnauthorized,
		},
		{
			desc:    "Unauthorized",
			request: &types.MsgUpdateL3Node{Creator: creator, Id: 10},
			err:     sdkerrors.ErrKeyNotFound,
		},
	} {
		t.Run(tc.desc, func(t *testing.T) {
			srv, ctx := setupMsgServer(t)
			_, err := srv.CreateL3Node(ctx, &types.MsgCreateL3Node{Creator: creator})
			require.NoError(t, err)

			_, err = srv.UpdateL3Node(ctx, tc.request)
			if tc.err != nil {
				require.ErrorIs(t, err, tc.err)
			} else {
				require.NoError(t, err)
			}
		})
	}
}

func TestL3NodeMsgServerDelete(t *testing.T) {
	creator := "A"

	for _, tc := range []struct {
		desc    string
		request *types.MsgDeleteL3Node
		err     error
	}{
		{
			desc:    "Completed",
			request: &types.MsgDeleteL3Node{Creator: creator},
		},
		{
			desc:    "Unauthorized",
			request: &types.MsgDeleteL3Node{Creator: "B"},
			err:     sdkerrors.ErrUnauthorized,
		},
		{
			desc:    "KeyNotFound",
			request: &types.MsgDeleteL3Node{Creator: creator, Id: 10},
			err:     sdkerrors.ErrKeyNotFound,
		},
	} {
		t.Run(tc.desc, func(t *testing.T) {
			srv, ctx := setupMsgServer(t)

			_, err := srv.CreateL3Node(ctx, &types.MsgCreateL3Node{Creator: creator})
			require.NoError(t, err)
			_, err = srv.DeleteL3Node(ctx, tc.request)
			if tc.err != nil {
				require.ErrorIs(t, err, tc.err)
			} else {
				require.NoError(t, err)
			}
		})
	}
}
