package keeper

import (
	"context"
	"fmt"

	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

func (k msgServer) CreateRollupDigest(goCtx context.Context, msg *types.MsgCreateRollupDigest) (*types.MsgCreateRollupDigestResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	var rollupDigest = types.RollupDigest{
		Creator:     msg.Creator,
		DsUri:       msg.DsUri,
		CurrentHash: msg.CurrentHash,
		Status:      msg.Status,
	}

	id := k.AppendRollupDigest(
		ctx,
		rollupDigest,
	)

	return &types.MsgCreateRollupDigestResponse{
		Id: id,
	}, nil
}

func (k msgServer) UpdateRollupDigest(goCtx context.Context, msg *types.MsgUpdateRollupDigest) (*types.MsgUpdateRollupDigestResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	var rollupDigest = types.RollupDigest{
		Creator:     msg.Creator,
		Id:          msg.Id,
		DsUri:       msg.DsUri,
		CurrentHash: msg.CurrentHash,
		Status:      msg.Status,
	}

	// Checks that the element exists
	val, found := k.GetRollupDigest(ctx, msg.Id)
	if !found {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, fmt.Sprintf("key %d doesn't exist", msg.Id))
	}

	// Checks if the msg creator is the same as the current owner
	if msg.Creator != val.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.SetRollupDigest(ctx, rollupDigest)

	return &types.MsgUpdateRollupDigestResponse{}, nil
}

func (k msgServer) DeleteRollupDigest(goCtx context.Context, msg *types.MsgDeleteRollupDigest) (*types.MsgDeleteRollupDigestResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	// Checks that the element exists
	val, found := k.GetRollupDigest(ctx, msg.Id)
	if !found {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, fmt.Sprintf("key %d doesn't exist", msg.Id))
	}

	// Checks if the msg creator is the same as the current owner
	if msg.Creator != val.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.RemoveRollupDigest(ctx, msg.Id)

	return &types.MsgDeleteRollupDigestResponse{}, nil
}
