package keeper_test

import (
	"testing"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	"github.com/stretchr/testify/require"
)

func createNRollupDigest(keeper *keeper.Keeper, ctx sdk.Context, n int) []types.RollupDigest {
	items := make([]types.RollupDigest, n)
	for i := range items {
		items[i].Id = keeper.AppendRollupDigest(ctx, items[i])
	}
	return items
}

func TestRollupDigestGet(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollupDigest(keeper, ctx, 10)
	for _, item := range items {
		got, found := keeper.GetRollupDigest(ctx, item.Id)
		require.True(t, found)
		require.Equal(t, item, got)
	}
}

func TestRollupDigestRemove(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollupDigest(keeper, ctx, 10)
	for _, item := range items {
		keeper.RemoveRollupDigest(ctx, item.Id)
		_, found := keeper.GetRollupDigest(ctx, item.Id)
		require.False(t, found)
	}
}

func TestRollupDigestGetAll(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollupDigest(keeper, ctx, 10)
	require.ElementsMatch(t, items, keeper.GetAllRollupDigest(ctx))
}

func TestRollupDigestCount(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollupDigest(keeper, ctx, 10)
	count := uint64(len(items))
	require.Equal(t, count, keeper.GetRollupDigestCount(ctx))
}
