package types

import (
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

var _ sdk.Msg = &MsgCreateL3Node{}

func NewMsgCreateL3Node(creator string, publicKey string, options []string) *MsgCreateL3Node {
	return &MsgCreateL3Node{
		Creator:   creator,
		PublicKey: publicKey,
		Options:   options,
	}
}

func (msg *MsgCreateL3Node) Route() string {
	return RouterKey
}

func (msg *MsgCreateL3Node) Type() string {
	return "CreateL3Node"
}

func (msg *MsgCreateL3Node) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgCreateL3Node) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgCreateL3Node) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgUpdateL3Node{}

func NewMsgUpdateL3Node(creator string, id uint64, publicKey string, options []string) *MsgUpdateL3Node {
	return &MsgUpdateL3Node{
		Id:        id,
		Creator:   creator,
		PublicKey: publicKey,
		Options:   options,
	}
}

func (msg *MsgUpdateL3Node) Route() string {
	return RouterKey
}

func (msg *MsgUpdateL3Node) Type() string {
	return "UpdateL3Node"
}

func (msg *MsgUpdateL3Node) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgUpdateL3Node) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgUpdateL3Node) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgDeleteL3Node{}

func NewMsgDeleteL3Node(creator string, id uint64) *MsgDeleteL3Node {
	return &MsgDeleteL3Node{
		Id:      id,
		Creator: creator,
	}
}
func (msg *MsgDeleteL3Node) Route() string {
	return RouterKey
}

func (msg *MsgDeleteL3Node) Type() string {
	return "DeleteL3Node"
}

func (msg *MsgDeleteL3Node) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgDeleteL3Node) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgDeleteL3Node) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}
