package types

import (
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

var _ sdk.Msg = &MsgCreateRollupDigest{}

func NewMsgCreateRollupDigest(creator string, dsUri string, currentHash string, status string) *MsgCreateRollupDigest {
	return &MsgCreateRollupDigest{
		Creator:     creator,
		DsUri:       dsUri,
		CurrentHash: currentHash,
		Status:      status,
	}
}

func (msg *MsgCreateRollupDigest) Route() string {
	return RouterKey
}

func (msg *MsgCreateRollupDigest) Type() string {
	return "CreateRollupDigest"
}

func (msg *MsgCreateRollupDigest) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgCreateRollupDigest) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgCreateRollupDigest) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgUpdateRollupDigest{}

func NewMsgUpdateRollupDigest(creator string, id uint64, dsUri string, currentHash string, status string) *MsgUpdateRollupDigest {
	return &MsgUpdateRollupDigest{
		Id:          id,
		Creator:     creator,
		DsUri:       dsUri,
		CurrentHash: currentHash,
		Status:      status,
	}
}

func (msg *MsgUpdateRollupDigest) Route() string {
	return RouterKey
}

func (msg *MsgUpdateRollupDigest) Type() string {
	return "UpdateRollupDigest"
}

func (msg *MsgUpdateRollupDigest) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgUpdateRollupDigest) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgUpdateRollupDigest) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgDeleteRollupDigest{}

func NewMsgDeleteRollupDigest(creator string, id uint64) *MsgDeleteRollupDigest {
	return &MsgDeleteRollupDigest{
		Id:      id,
		Creator: creator,
	}
}
func (msg *MsgDeleteRollupDigest) Route() string {
	return RouterKey
}

func (msg *MsgDeleteRollupDigest) Type() string {
	return "DeleteRollupDigest"
}

func (msg *MsgDeleteRollupDigest) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgDeleteRollupDigest) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgDeleteRollupDigest) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}
